# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.interface import Interface, InterfaceMeta

import enum
import six


@six.add_metaclass(InterfaceMeta)
class DataSource(Interface):
    """Offer to transfer data

    The :class:`DataSource` object is the source side of a :class:`~pywayland.protocol.dataoffer.DataOffer`.
    It is created by the source client in a data transfer and
    provides a way to describe the offered data and a way to respond
    to requests to transfer the data.
    """
    name = "wl_data_source"
    version = 3

    error = enum.Enum("error", {
        "invalid_action_mask": 0,
        "invalid_source": 1,
    })


@DataSource.request("s", [None])
def offer(self, mime_type):
    """Add an offered mime type

    This request adds a mime type to the set of mime types
    advertised to targets.  Can be called several times to offer
    multiple types.

    :param mime_type: mime type offered by the data source
    :type mime_type: `string`
    """
    self._marshal(0, mime_type)


@DataSource.request("", [])
def destroy(self):
    """Destroy the data source

    Destroy the data source.
    """
    self._marshal(1)
    self._destroy()


@DataSource.request("3u", [None])
def set_actions(self, dnd_actions):
    """Set the available drag-and-drop actions

    Sets the actions that the source side client supports for this
    operation. This request may trigger :func:`DataSource.action` and
    :func:`DataOffer.action() <pywayland.protocol.dataoffer.DataOffer.action>` events if the compositor needs to change the
    selected action.

    The dnd_actions argument must contain only values expressed in the
    :func:`DataDeviceManager.dnd_actions() <pywayland.protocol.datadevicemanager.DataDeviceManager.dnd_actions>` enum, otherwise it will result
    in a protocol error.

    This request must be made once only, and can only be made on sources
    used in drag-and-drop, so it must be performed before
    :func:`DataDevice.start_drag() <pywayland.protocol.datadevice.DataDevice.start_drag>`. Attempting to use the source other than
    for drag-and-drop will raise a protocol error.

    :param dnd_actions: actions supported by the data source
    :type dnd_actions: `uint`
    """
    self._marshal(2, dnd_actions)


@DataSource.event("?s", [None])
def target(self, mime_type):
    """A target accepts an offered mime type

    Sent when a target accepts pointer_focus or motion events.  If
    a target does not accept any of the offered types, type is NULL.

    Used for feedback during drag-and-drop.

    :param mime_type: mime type accepted by the target
    :type mime_type: `string` or `None`
    """
    self._post_event(0, mime_type)


@DataSource.event("sh", [None, None])
def send(self, mime_type, fd):
    """Send the data

    Request for data from the client.  Send the data as the
    specified mime type over the passed file descriptor, then
    close it.

    :param mime_type: mime type for the data
    :type mime_type: `string`
    :param fd: file descriptor for the data
    :type fd: `fd`
    """
    self._post_event(1, mime_type, fd)


@DataSource.event("", [])
def cancelled(self):
    """Selection was cancelled

    This data source is no longer valid. There are several reasons why
    this could happen:

    - The data source has been replaced by another data source.
    - The drag-and-drop operation was performed, but the drop destination
    did not accept any of the mime types offered through
    :func:`DataSource.target`.
    - The drag-and-drop operation was performed, but the drop destination
    did not select any of the actions present in the mask offered through
    :func:`DataSource.action`.
    - The drag-and-drop operation was performed but didn't happen over a
    surface.
    - The compositor cancelled the drag-and-drop operation (e.g. compositor
    dependent timeouts to avoid stale drag-and-drop transfers).

    The client should clean up and destroy this data source.

    For objects of version 2 or older, :func:`DataSource.cancelled` will
    only be emitted if the data source was replaced by another data
    source.
    """
    self._post_event(2)


@DataSource.event("3", [])
def dnd_drop_performed(self):
    """The drag-and-drop operation physically finished

    The user performed the drop action. This event does not indicate
    acceptance, :func:`DataSource.cancelled` may still be emitted afterwards
    if the drop destination does not accept any mime type.

    However, this event might however not be received if the compositor
    cancelled the drag-and-drop operation before this event could happen.

    Note that the data_source may still be used in the future and should
    not be destroyed here.
    """
    self._post_event(3)


@DataSource.event("3", [])
def dnd_finished(self):
    """The drag-and-drop operation concluded

    The drop destination finished interoperating with this data
    source, so the client is now free to destroy this data source and
    free all associated data.

    If the action used to perform the operation was "move", the
    source can now delete the transferred data.
    """
    self._post_event(4)


@DataSource.event("3u", [None])
def action(self, dnd_action):
    """Notify the selected action

    This event indicates the action selected by the compositor after
    matching the source/destination side actions. Only one action (or
    none) will be offered here.

    This event can be emitted multiple times during the drag-and-drop
    operation, mainly in response to destination side changes through
    :func:`DataOffer.set_actions() <pywayland.protocol.dataoffer.DataOffer.set_actions>`, and as the data device enters/leaves
    surfaces.

    It is only possible to receive this event after
    :func:`DataSource.dnd_drop_performed` if the drag-and-drop operation
    ended in an "ask" action, in which case the final :func:`DataSource.action`
    event will happen immediately before :func:`DataSource.dnd_finished`.

    Compositors may also change the selected action on the fly, mainly
    in response to keyboard modifier changes during the drag-and-drop
    operation.

    The most recent action received is always the valid one. The chosen
    action may change alongside negotiation (e.g. an "ask" action can turn
    into a "move" operation), so the effects of the final action must
    always be applied in :func:`DataOffer.dnd_finished() <pywayland.protocol.dataoffer.DataOffer.dnd_finished>`.

    Clients can trigger cursor surface changes from this point, so
    they reflect the current action.

    :param dnd_action: action selected by the compositor
    :type dnd_action: `uint`
    """
    self._post_event(5, dnd_action)


DataSource._gen_c()
